/*jshint smarttabs:true, eqeqeq:false, eqnull:true, laxbreak:true*/
/**
 * @author kongxiangxu
 * Interact with native application services
 */
(function(window, vx, undefined) {'use strict';

	var service = {};
	service.$nativeCall = ["$os", "$log", '$rootScope', '$targets',
	function($os, $log, $rootScope, $targets) {
		var tNative = {
			csii__index : 1,
			csii__start : 1,
			csii__data : [],		//params transfer to native
			csii__callback : [],	//callback function transfer to native
			history:[],
			viewPort:'content',
			rootScope : $rootScope,
			isHideBackButton: false,
            ShowMasking: false,
            timer: null
		};
		var loadUrl = function(url){
			var iFrame = document.createElement("iframe");
		    iFrame.setAttribute("src", url);
		    iFrame.setAttribute("style", "display:none;");
		    iFrame.setAttribute("height", "0px");
		    iFrame.setAttribute("width", "0px");
		    iFrame.setAttribute("frameborder", "0");
		    document.body.appendChild(iFrame);
		    iFrame.parentNode.removeChild(iFrame);
		    iFrame = null;
		};
		/**
		 * call native fn
		 */
		var nativeCall = function(msg, callback) {
			$log.debug("init native call " + msg);
			var params, url;
			if( typeof callback === "undefined" || callback === undefined) {
				if($os.android) {
					CSII.NativeCall(msg);
				} else if($os.ios) {
					url = "file://localhost/LocalActions/"+ msg;
					loadUrl(url);
				} else if($os.wphone){
					params = {
						"command" : msg
					};
					window.external.notify(vx.toJson(params));
				}
			} else if(arguments.length < 3) {
				tNative.csii__callback[tNative.csii__index] = callback;
				if($os.android) {
					CSII.NativeCall(msg, tNative.csii__index);
				} else if($os.ios) {
					url = "file://localhost/LocalActions/"+ msg + '___' + tNative.csii__index;
					loadUrl(url);
				} else if($os.wphone){
					params = {
						"command" : msg,
						"index" : tNative.csii__index
					};
					window.external.notify(vx.toJson(params));
				}
				tNative.csii__index++;
			} else {
				var start = tNative.csii__start, i;
				for(i = 2; i < arguments.length; i++) {
					tNative.csii__data[tNative.csii__start++] = arguments[i];
				}
				tNative.csii__callback[tNative.csii__index] = callback;
				if($os.android) {
					CSII.NativeCall(msg, tNative.csii__index, start);
				} else if($os.ios) {
					url = "file://localhost/LocalActions/"+ msg + '___' + tNative.csii__index + '___' + start;
					loadUrl(url);
				} else if($os.wphone){
					var tmp, tmpIndex;
					if(msg === "SendRequest") {
						tmp = arguments[3];
						tmpIndex = start;
					} else {
						tmp = arguments[2];
						tmpIndex = tNative.csii__index;
					}
					if(tmp.match(/^{.+}$/) || tmp.match(/^[.+]$/)) {
						params = {
							"command" : msg,
							"index" : tmpIndex,
							"data" : vx.fromJson(tmp)
						};
					} else {
						params = {
							"command" : msg,
							"index" : tmpIndex,
							"data" : tmp
						};
					}
					window.external.notify(vx.toJson(params));
				}
				tNative.csii__index++;
			}
		};
		/**
		 * android call javascript interface
		 */
		tNative.androidGetValue = function(str){
			eval(str);
		};
		
		
		/**
		 * set the native app whil title
		 */
		tNative.setTitle = function(title) {
			//var title = this.activePage.attr("title");
			if(title){
				nativeCall("SetTitle", null, title);
			}
		};
		/**
		 * set the native app back of button show or hide
		 */
		tNative.setBackButtonVisibility = function(flag) {
			if(flag){
				if(!tNative.isHideBackButton){
					tNative.isHideBackButton = true;
					nativeCall("HideBackButton");
				}
			}else if(tNative.isHideBackButton){
				nativeCall("ShowBackButton");
				tNative.isHideBackButton = false;
			}
		};
		/**
		 * call back native app show loading window
		 */
		tNative.showMask = function() {
			nativeCall("ShowMask");
		};
		/**
		 * call back native app hide loading window
		 */
//		tNative.hideMask = function() {
//			window.setTimeout(function(){nativeCall("HideMask");}, 500);
//		};
        tNative.hideMask = function() {
                           nativeCall("HideMask");
                           };
		/**
		 * close native app splash screen
		 */
		tNative.closeSplashScreen = function() {
			nativeCall("CloseSplashScreen");
		};
		tNative.alert = function(message) {
			nativeCall("Alert", null, message);
		};

		tNative.confirm = function(callback, message) {
			nativeCall("Confirm", function(yes) {
				if($rootScope.$$phase) {
					callback(yes);
				} else {
					$rootScope.$apply(function() {
						callback(yes);
					});
				}
			}, message);
			//TODO FOR DEV
			/*if($rootScope.$$phase) {
				callback(true);
			} else {
				$rootScope.$apply(function() {
					callback("Yes");
				});
			}*/
			//TODO END
		};

		tNative.toast = function(errMessage) {
			$log.debug("Toast:" + errMessage);
			nativeCall("Toast", null, errMessage);
		};
		
		/**
		 * 通知客户端获取公钥
		 */
		tNative.publicKeyModulus = function(callback) {
			nativeCall("PublicKeyModulus", function(data){
//				tNative.toast("data : " + data);
				if(data == "")return;
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					if(callback){
						callback(data);
					}
				});
			});
		};
		
		/*
		 *vx传预留信息
		 */
		tNative.sendWeclomeInfo = function(errMessage) {
			nativeCall("SendWeclomeInfo", null, errMessage);
		};
		
		/*
		 * open a website
		 */
		tNative.openWebsite = function(url){
			nativeCall("OpenWebsite", null, url);
		};
		
		tNative.datePicker = function(callback, date) {
			nativeCall("DatePicker", callback, date);
		};

		/**
		 * call native app authenticate module
		 */
		tNative.getPhoneNumber = function(callback){
			nativeCall("GetPhoneNumber", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					var PhoneNumber = data.PhoneNumber;
					if(PhoneNumber){
						//去掉手机号里面里面的空格、-
						data.PhoneNumber = data.PhoneNumber.replace(/\s|-/g, '');
					}
					callback(data);
				});
			});
		};
		
		/**
		 * 请求Android那边数据
		 * call native app authenticate module
		 */
		tNative.getLoginUserInfo = function(callback){
			nativeCall("GetLoginUserInfo", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		
		/**
		 * 用户登出
		 */
		tNative.userLogout = function(callback){
			nativeCall("UserLogout", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		
		/**
		 * 重新登录
		 */
		tNative.reLogin = function(callback){
			nativeCall("ReLogin", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		
		/**
		 * 两次支付密码判断
		 */
		tNative.bothPwdVerify = function(callback){
			nativeCall("BothPwdVerify", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		/**
		 * 两次取款密码判断
		 */
		tNative.bothPwdVerifyV1 = function(callback){
			nativeCall("BothPwdVerifyV1", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		
		/**
		 * 两次取票密码判断
		 */
		tNative.bothPwdVerifyV2 = function(callback){
			nativeCall("BothPwdVerifyV2", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
				});
			});
		};
		
		/**支付密码
		 * call native app authenticate module
		 */
		tNative.authenticate = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("Authenticate", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		/**确认支付密码
		 * call native app authenticate module
		 */
		tNative.authenticateConfirm = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateConfirm", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
				tNative.authenticateConfirmV8 = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateConfirmV8", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
		/**取款密码
		 * call native app authenticate module
		 */
		tNative.authenticateV1 = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateV1", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
		/**确认取款密码
		 * call native app authenticate module
		 */
		tNative.authenticateConfirmV1 = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateConfirmV1", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
		/**汽车票取票密码
		 * call native app authenticate module
		 */
		tNative.authenticateV2 = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateV2", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
		/**确认汽车票取票密码
		 * call native app authenticate module
		 */
		tNative.authenticateConfirmV2 = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("AuthenticateConfirmV2", callback);
			}else{
				callback(vx.toJson({"TrsPassword":"111111"}));
			}			
		};
		
		/**
		 * 获取短信，得到验证码
		 */
		tNative.getSmsCode = function(callback){
			nativeCall("GetSmsCode", callback);
		};
		
		/**
		 * 系统参数
		 */
		tNative.getSystemVar = function(callback) {
			nativeCall("GetSystemVar", function(data){
				$rootScope.$apply(function(){
					data = vx.fromJson(data);
					callback(data);
//				tNative.toast("data.smsTimeout:" + data.smsTimeout);
				});
			});		
		};
		
		//UK
		tNative.authenticateUK = function(callback,data2xml) {
			nativeCall("AuthenticateUK", callback, data2xml);		
		};
		//交易密码和证书
		tNative.authenticateForUK = function(callback,data2xml) {
			nativeCall("AuthenticateForUK", callback, data2xml);		
		};		
		//登录密码和证书
		tNative.authenticateLoginAndUK = function(callback,data2xml) {
			nativeCall("AuthenticateLoginAndUK", callback, data2xml);		
		};
		//获取飞机票订单数据
		tNative.getAirOrderData = function(callback){
			nativeCall("GetAirOrderData", function(data){
//				$rootScope.$apply(function(){
//					data = vx.fromJson(data);
					callback(data);
//				});	
			});
		};
		/**
		 * call native app authenticate module
		 */
		tNative.authenticateLogin = function(callback) {
			if($os.ios || $os.android || $os.wphone){
				nativeCall("Loginpassword", callback);
			}else{
				callback(vx.toJson({"TrsPassword":""}));
			}
		};		
		/**
		 * get set data
		 */
		tNative.setAccountProtection=function(flag){
			nativeCall("SetAccountProtection",null,flag);
		};			
		/**
		 * close native app webview
		 */
		tNative.finishWeb = function() {
			nativeCall("FinishWeb");
		};
		/**
		 * close native app webview
		 */
		tNative.finishWebWithMessage = function(mes) {
			nativeCall("FinishWebWithMessage", null, mes);
		};		
		/**
		 * close native app webview
		 * Yolen 2013-9-25
		 */
		tNative.gotoMenu = function(menuName){
			nativeCall("GotoMenu", null, menuName);
		};

		/**
		 * notice native app will transit forward
		 */
		tNative.forWardTransition = function() {
			nativeCall("ForWardTransition");
		};

		/**
		 * notice native app will transit back
		 */
		tNative.backTransition = function() {
			nativeCall("BackTransition");
		};

		/**
		 * notice native app do transition animation
		 */
		tNative.startTransition = function() {
			nativeCall("StartTransition");
		};
		
		/**
		 * invoke iphone custom keyboard 
		 */
		tNative.changeKeyboard = function() {
			nativeCall("ChangeKeyboard");
		};

		/**
		 * setting viewport name 
		 */
		tNative.setViewPortName = function(name) {
			tNative.viewPort = name || 'content';
		};
		
		/**
		 * native app call html app goback fn
		 */
		tNative.goBack = function() {
		

		
			if(tNative.history.length <= 1){
				//call native goback fn
				return "false";
			} else if( typeof tNative.history[tNative.history.length -1] !== 'function'){
				var vLength = tNative.history.length;
				var vLast = tNative.history[vLength-1];
				var vPenult = tNative.history[vLength-2];
				$targets(tNative.viewPort, "#" + (vPenult-vLast));
				return "true";
			} else if(typeof tNative.history[tNative.history.length -1] === 'function'){	//回调函数
				var callback = tNative.history.pop();
				callback();
				return "true";
			}
		};
		
		/**
		 * native app call html app,open transfer 
		 */
		tNative.loadTrasfer=function(viewport,url){
			$rootScope.$apply(function(){
				$targets(viewport,url);
			});
		};
		
		tNative.getUserInfo = function(callback){
			nativeCall("GetUserInfo", callback);
		};
		
		tNative.sendRequest = function(request, params){
			nativeCall("SendRequest", null, request, params);
		};
		
		tNative.getActionId = function(callback){
			nativeCall("GetActionId",callback);
		};
		
		tNative.csii__InputContent = function(str, inputId){
			var target = vx.element("#" + inputId);
			target.val(str);
			target.trigger("input");
		};
		
		tNative.csii__InputData = function(inputStr) {
		    //TODO:Keyboard
		    if (inputStr === "delete") {
		        if (document.activeElement.selectionStart - document.activeElement.selectionEnd == 0) {
		            var valueStr = document.getElementById(document.activeElement.id).value;
		            valueStr = valueStr.substring(0, document.activeElement.selectionStart - 1) + valueStr.substring(document.activeElement.selectionStart, valueStr.length);
		            var selectionStart = document.activeElement.selectionStart;
		            document.getElementById(document.activeElement.id).value = valueStr;
		            document.activeElement.selectionStart = selectionStart - 1;
		            document.activeElement.selectionEnd = selectionStart - 1;
		        } else {
		            var valueStr = document.getElementById(document.activeElement.id).value;
		            valueStr = valueStr.substring(0, document.activeElement.selectionStart) + valueStr.substring(document.activeElement.selectionEnd, valueStr.length);
		            var selectionStart = document.activeElement.selectionStart;
		            document.getElementById(document.activeElement.id).value = valueStr;
		            if (selectionStart == 0) {
		                document.activeElement.selectionEnd = 0;
		            }else{
		                document.activeElement.selectionEnd = document.activeElement.selectionStart;
		            }
		        }
		    } else {
		        if (document.activeElement.selectionStart - document.activeElement.selectionEnd == 0) {
		            var valueStr = document.getElementById(document.activeElement.id).value;
		            valueStr = valueStr.substring(0, document.activeElement.selectionStart) + inputStr + valueStr.substring(document.activeElement.selectionStart, valueStr.length);
		            var selectionStart = document.activeElement.selectionStart;
		            document.getElementById(document.activeElement.id).value = valueStr;
		            document.activeElement.selectionStart = selectionStart + 1;
		            document.activeElement.selectionEnd = selectionStart + 1;
		        } else {
		            var valueStr = document.getElementById(document.activeElement.id).value;
		            valueStr = valueStr.substring(0, document.activeElement.selectionStart) + inputStr + valueStr.substring(document.activeElement.selectionEnd, valueStr.length);
		            var selectionStart = document.activeElement.selectionStart;
		            document.getElementById(document.activeElement.id).value = valueStr;
		            document.activeElement.selectionStart = selectionStart + 1;
		            document.activeElement.selectionEnd = document.activeElement.selectionStart;
		        }
		    }
		    var target = document.getElementById(document.activeElement.id);
		    vx.element(target).trigger("input");
		};
		
		return tNative;
	}];


	vx.module('mapp.libraries').service(service);

})(window, window.vx);

/*jshint smarttabs:true, eqeqeq:false, eqnull:true, laxbreak:true*/
/**
 * @author kongxiangxu
 * to obtain equipment services
 */
(function(window, vx, userAgent) {
	'use strict';
	
	var service = {};
	service.$os = [ function() {
		var os = {
			webkit : userAgent.match(/WebKit\/([\d.]+)/) ? true : false,
			android : userAgent.match(/(Android)\s+([\d.]+)/) || userAgent.match(/Silk-Accelerated/) ? true : false,
			androidICS : this.android && userAgent.match(/(Android)\s4/) ? true : false,
			ipad : userAgent.match(/(iPad).*OS\s([\d_]+)/) ? true : false,
			iphone : !(userAgent.match(/(iPad).*OS\s([\d_]+)/) ? true : false) && userAgent.match(/(iPhone\sOS)\s([\d_]+)/) ? true : false,
			ios : (userAgent.match(/(iPad).*OS\s([\d_]+)/) ? true : false) || (!(userAgent.match(/(iPad).*OS\s([\d_]+)/) ? true : false) && userAgent.match(/(iPhone\sOS)\s([\d_]+)/) ? true : false),
			ios5 : (userAgent.match(/(iPad).*OS\s([5_]+)/) ? true : false) || (!(userAgent.match(/(iPad).*OS\s([5_]+)/) ? true : false) && userAgent.match(/(iPhone\sOS)\s([5_]+)/) ? true : false),
			wphone : userAgent.match(/Windows Phone/i)?true:false
		};
		return os;
	} ];

	vx.module('mapp.libraries').service(service);

})(window, window.vx,navigator.userAgent);
/*jshint smarttabs:true, eqeqeq:false, eqnull:true, laxbreak:true*/
/**
 * @author kongxiangxu
 * transition services
 */
(function(window, vx, userAgent) {
	'use strict';
	
	var service = {};
	service.$transitions = ["$nativeCall","$os", function($nativeCall,$os) {
		var transitionProvider = {
			types : {
				'native' : 'native',
				'default' : 'default',
				'none' : 'none'
			},
			runTransition : runTransition,
			availableTransitions:{}
		};
		
		function runTransition(transition, oldDiv, currWhat, remove,back) {
			
			if(!transitionProvider.availableTransitions[transition])
				transition = 'default';
			transitionProvider.availableTransitions[transition].call(this, oldDiv, currWhat, remove,back);
		}
		
		(function(transitionProvider,$nativeCall){
			
			function nativeTransition(oldEl, newEl, remove, back) {
				
				/**
				 * get vx viewport active index value
				 */
				var controller = newEl.controller("vViewport");
				var activeIndex = controller.$pages.activeIndex;
				
				/**
				 * add form attributes what auto and check
				 */
				var forms = newEl.find("form");
				forms.attr("autocomplete", "off");
				forms.attr("autocorrect", "off");
				forms.attr("autocapitalize", "off");
				forms.attr("spellcheck", "off");
				
				/*controller.$pages.track.length === activeIndex */
				if(newEl.attr("data-back")==="false") {//跳转到结果页
					$nativeCall.history = [];
					$nativeCall.setBackButtonVisibility(true);
				} else {
					for(var ii in $nativeCall.history){
						if($nativeCall.history[ii] === activeIndex){
							$nativeCall.history = $nativeCall.history.slice(0,ii);
							break;
						}
					}
					if(newEl.attr("history")!=="false"){
						$nativeCall.history.push(activeIndex);
					}
				}
				if(oldEl !== null){
					if(!back) {
						//This method needs to be changed
						$nativeCall.forWardTransition();
					} else {
						//This method needs to be changed
						$nativeCall.backTransition();
					}
					
					window.setTimeout(function(){
				        if(newEl && newEl.length) {
					        if(oldEl && oldEl.length) {
						        oldEl.css('display', 'none');
						        if(remove)
		        				    oldEl.remove();
					        }
					        	newEl.css('display', 'block');
				            }
				        finishTransition(oldEl, newEl, remove, back);
			        },200);
				} else {
					newEl.css("display","block");
					$nativeCall.closeSplashScreen();
				}
			}
			
			function finishTransition(oldEl, newEl, remove, back) {
				//this.activePage = newEl;
				$nativeCall.setTitle(newEl.attr("title"));
				scrollToTop();
				$nativeCall.startTransition();
			}

			function scrollToTop() {
				window.scrollTo(0, 0);
			}

			transitionProvider.availableTransitions['native'] = nativeTransition;
		})(transitionProvider,$nativeCall);

		return transitionProvider;
	} ];

	vx.module('mapp.libraries').service(service);

})(window, window.vx);


/**
 * @author kongxiangxu
 */
(function(window, vx, undefined) {'use strict';

	var service = {};
	service.Util = ['$filter',
	function($filter) {
		return {
			/**
	         * description:还回时间格式类似为"2012-05-16"字符串, format指定字符串格式， 默认yyyy-MM-dd.
	         * example:getDate()还回当前时间的，getDate("3d")三天前时间，getDate("3w")三周前的，getDate("3m")三个月前的
	         */
			getDate : function(days, format) {
				// TODO 添加函数过程
				format = format || 'yyyy-MM-dd';
				if (days) {
					var group = days.match(/(\d+)([dDMmWw])/);
					var value = group[1], type = group[2].toUpperCase();
					if (type === 'D')
						return $filter('date')(new Date(new Date().getTime() - (value * 24 * 3600 * 1000)), format);
					else if (type === 'W')
						return $filter('date')(new Date(new Date().getTime() - (value * 7 * 24 * 3600 * 1000)), format);
					else if (type === 'M') {
						var date = new Date();
						date.setMonth(date.getMonth() - value);
						return $filter('date')(date, format);
					}
				} else
					return $filter('date')(new Date(), format);
			},
			/**
			 * 加密账号 加密后形式1234****5678
			 */
			encryptAcNo: function(acno){
				var length = acno.length;
				return acno.substring(0,4) + "****" + acno.substring(length-4, length);
			},
			daysBetween: function(beginDate,endDate){
				var OneMonth = beginDate.substring(5,beginDate.lastIndexOf ('-'));  
				var OneDay = beginDate.substring(beginDate.length,beginDate.lastIndexOf ('-')+1);  
				var OneYear = beginDate.substring(0,beginDate.indexOf ('-'));  
				var TwoMonth = endDate.substring(5,endDate.lastIndexOf ('-'));  
				var TwoDay = endDate.substring(endDate.length,endDate.lastIndexOf ('-')+1);  
				var TwoYear = endDate.substring(0,endDate.indexOf ('-'));  
				var difference=((Date.parse(TwoMonth+'/'+TwoDay+'/'+TwoYear)- Date.parse(OneMonth+'/'+OneDay+'/'+OneYear))/86400000);
				return difference;
			},
			/**
			 * 比较金额前格式化金额
			 */
			parseAmount: function(amount){
				amount = amount ? amount.replace(/,/g, "") : 0;
				return parseFloat(amount);
			},
			/**
			 * 保留两位小数
			 */
			moneyFormat:function(amount){
				amount = amount + "";
				amount = amount ? amount.replace(/,/g, "") : 0;
				amount = amount * 1;
				amount = amount.toFixed(2);
				return amount;
			}
			
		};
	}];

	vx.module('mapp.libraries').service(service);

})(window, window.vx);

/**
 * @author 
 */
(function(window, vx, undefined) {
	'use strict';
	vx.module('mapp.libraries').factory("$goto", ['$targets','$nativeCall', function($targets, $nativeCall) {
		return function(url){
			if(url.indexOf('#') > -1){
				$targets("content", url);
			}else {
				if($nativeCall.isHideBackButton){
					$nativeCall.setBackButtonVisibility(false);
				}
				if(url.indexOf(".html") < 0)
					url = "htmls/"+url+"/"+url+".html";
				$targets("content", url);
			}
		};
	}]);

})(window, window.vx);

/*
 * sessionService
 */
(function(window, vx, undefined) { 'use strict';

var service = {};
service.sessionService = [ function() {
	return {
		params : {}
	};
} ];

vx.module('mapp.libraries').service(service);

})(window, window.vx);