/*jshint smarttabs:true, eqeqeq:false, eqnull:true, laxbreak:true*/

/**
 * mobile app config
 */
vx.module('mapp.config', []).value('mapp.config', {});
vx.module('mapp.libraries', ['mapp.config']);
vx.module('mapp', ['mapp.libraries', 'ui']);

/**
 * mobile ui config
 */
vx.module('ui.config', []).value('ui.config', {});
vx.module('ui.libraries', ['ui.config']);
vx.module('ui', ['ui.libraries', 'ui.config']);

/**
 *  Example Source Code Config
 */
(function(window, vx, $) {
	'use strict';
	/**
	 * this block is module config, if you want do some module management, please use vx-plugins.js and manage module by following methods:
	 */
	// **  module.provider(...) / module.factory(...) / module.service(...) / module.value(...) / module.constant(...)
	// **  module.filter(...)
	// **  module.directive(...)
	// **  module.controller(...)

	//### Configuration Entry
	var mod = vx.module('mapp.config');

	/************************************************
	 * config service factory function
	 ************************************************/
	//Log配置
	configLog.$inject = ['$logProvider'];
	function configLog($logProvider) {
		/**
		 * log level config, support 'debug', 'info', 'warn', 'error'
		 *  note: $log === window.console
		 *  if IE 6/7, include blackbird.js and blackbird.css will emulate window.console for you
		 *  default is 'debug'
		 */
		$logProvider.setLevel('info');
	}
	
	//Filter配置
	configFilter.$inject = ['$filterProvider'];
	function configFilter($filterProvider){
		vx.module('mapp').filterProvider = $filterProvider;
	}

	//Browser配置
	configBrowser.$inject = ['$browserProvider'];
	function configBrowser($browserProvider) {

		/**
		 * if E2ETest (end to end test), you should disable browser.debounce function
		 * so setE2ETest(true), debounce used to combind events handle for performance
		 *  default is false
		 *  $browserProvider.setE2ETest(false);
		 */

		/**
		 * config Low version of the browser returns no refresh,setting iframe history href initial value.
		 * default file name by blank.html
		 * $browserProvider.setBlankPage("empty.html");
		 */
	}

	//Targets配置
	configTargets.$inject = ['$targetsProvider'];
	function configTargets($targetsProvider) {
		/**
		 *  lets $targets service use window.History for browser forward and backward.
		 *  default is false
		 */
		$targetsProvider.useLocation(false);

		/**
		 * register transition function to $targets service,
		 *  transition function signature is function(oldEl, newEl, remove, back)
		 *  $targetsProvider.transition('transition-name', function(oldEl, newEl, remove, back){});
		 */
	}

	//Compile配置
	configCompile.$inject = ['$compileProvider'];
	function configCompile($compileProvider) {
		/**
		 *  when vx set <a href='...' />, it will sanitize for avoid XSS attack
		 *  default is /^\s*(https?|ftp|mailto|file):/
		 *  $compileProvider.urlSanitizationWhitelist(/^\s*(https?|ftp|mailto|file):/);
		 */
	}
	//RootScope配置
	configRootScope.$inject = ['$rootScopeProvider'];
	function configRootScope($rootScopeProvider) {
		/**
		 * scope's digest is dirty loop, until now modification found, so loop count(TTL)
		 * is fatal for performance, it means loop count over TTL, the digest will exit
		 * even if has more modifications
		 *  default is 10
		 *  $rootScopeProvider.digestTtl(10);
		 */

		/**
		 * for scope digest analysis, $rootScope service will use log.debug tracing
		 *  digest processing,
		 *  default is false
		 */
		$rootScopeProvider.traceDigest(false);
	}

	//Remote配置
	configRemote.$inject = ['$$remoteProvider'];
	function configRemote($$remoteProvider) {
		/**
		 * $remote will use this name for scope, for example, scope.$error will get error object
		 *  default is '$error'
		 */
		//$$remoteProvider.setErrorTag('$error');

		/**
		 * $remote will use this callback analysis error, for examples
		 *  1.  status not in [200, 300), return 'http error'
		 *  2.  application data include $jsonError property will means application error
		 *
		 *  NOTE: application should provide this callback
		 */
		$$remoteProvider.setErrorCallback(function(data, status, headers, config) {
			// if error return error object, otherwise return null
			//防重复提交，如果_tokenName不为空。
			if (!vx.isEmpty(data._tokenName)) {
				config.$scope.$parent._tokenName = data._tokenName;
			}
			var $S = config.$scope, errorList = [], msg = "";
			if (!vx.isEmpty(status) && status != 200 && status !== 0) {
				switch(status) {
					case 404 :
						console.log("404.html");
						break;
					case 500 :
						console.log("500.html");
						break;
					default :
						console.log("default.html");
				}
			} else if (data && ((data.returnCode!="000000000") || !vx.isEmpty(data.jsonError)) && ("ER9999999" != data.returnCode)){
				if (data.returnMsg || data.jsonError) {
					if (vx.isArray(data.returnMsg) || vx.isArray(data.jsonError)) {
						errorList = !vx.isEmpty(data.returnMsg) ? data.returnMsg : data.jsonError;
					} else {
						msg = !vx.isEmpty(data.returnMsg) ? data.returnMsg : data.jsonError;
					}
				}
			}
			
			if (errorList) {
				for (var i = 0; i < errorList.length; i++) {
					if (errorList[i]._exceptionMessage) {
						msg = msg.concat(errorList[i]._exceptionMessage);
					} else {
						msg = msg.concat(errorList[i]);
					}
				}
			}
			if (msg) {
				var errorMsg;
				if (vx.isArray(msg)) {
					errorMsg = msg[0];
				} else {
					errorMsg = msg;
				}
				NativeCall.toast(msg);
				
				/**
				 * 需要上送密码的交易，交易失败后需要重新获取公钥
				 * P888888:支付，
				 * P202301:请车票订购
				 */
				var sendPwdTransCodes = {
						"P888888":"1",
						"P200406":"1",
						"P200407":"1",
						"P280204":"1",
						"P200417":"1",
						"P221001":"1",
						"P220012":"1",
						"P221002":"1"
				};
				if(sendPwdTransCodes[data.transCode] == "1"){
					msg = "";
				}else{
					
				}
//				config.$scope.$parent.clearTimeouts("submitPhone");
				return msg;
			}
		});

		/**
		 * $remote could add success callback by yourself, it will could be call later
		 *  like:   $remote.get(...).method(arg1, arg2)   ** first arg is data
		 *  you could extend $remote success callback by this
		 */
//		 $$remoteProvider.addCallback('error', function(data,callback) {
//			 
//			 scope.toast(1111111111111);
//			 scope.toast(data.returnCode);
//		 });
	}

	//Http配置
	configHttp.$inject = ['$httpProvider'];
	function configHttp($httpProvider) {
		/**
		 * config $http service if use inner cache(not HTTP cache), if use inner cache
		 * all same url GET will just submit server only once
		 * default is false
		 */
		//$httpProvider.useCache(false);

		/**
		 * config $http if use json request type, if not, use form encoding, for examples
		 *  abc=a&aaa=23
		 *  defautl is true
		 */
		//$httpProvider.useJsonRequest(true);

		/**
		 * config $http service default callback, if return true, it will disable all success and error callback
		 *  typically, this callback could use for session control
		 *  but in application use $remote is recommend, so you could use $remoteProvider.setErrorCallback for
		 *  same situation
		 */
		$httpProvider.setDefaultCallback(function(data, status, headers, config) {
			// if return true, means no need further process, all others success(...) and error(...) will not invoke
			return false;
		});

		/**
		 * config $http service defaults, you could:
		 * 1. $httpProvicer.defautls.transformResponse(array) override default json convert or add your function
		 * 2. $httpProvicer.defautls.transformRequest(array) override default json convert or add your function
		 * 3. $httpProvicer.defautls.headers define default HTTP Headers, default is
		 * {
		 *   common : {
		 *     'Accept' : 'application/json, text/plain, *\/*'
		 *   },
		 *   post : {
		 *     'Content-Type' : 'application/json;charset=utf-8'
		 *   },
		 *   put : {
		 *     'Content-Type' : 'application/json;charset=utf-8'
		 *   },
		 *   xsrfCookieName : 'XSRF-TOKEN',
		 *   xsrfHeaderName : 'X-XSRF-TOKEN'
		 * }
		 */
		//$httpProvider.defaults

		/**
		 * config $http service response interceptors, default is empty array
		 *  you could use  $httpProvider.responseInterceptors.push(fn(promise)) for add interceptor
		 *  promise is $q.defer.promise object, use then(success(...), error(...)) for register callback
		 */
		//$httpProvider.responseInterceptors
	}

	//HttpBackend配置
	configHttpBackend.$inject = ['$httpBackendProvider'];
	function configHttpBackend($httpBackendProvider) {

		/**
		 * config $httpBackend use anti-cache policy for HTTP cache
		 * 0-none, 1-use load timestamp, 2-use request timestamp
		 *  defautl is 0
		 */
		$httpBackendProvider.useAntiCache(0);

		/**
		 * config $httpBackend use external ajax function
		 * if true will use jQuery's ajax, otherwise use vx internal ajax
		 * default is false
		 */
		$httpBackendProvider.useExternalAjax(false);

		// config ajax default port
		$httpBackendProvider.config({
			/**
			 * config $httpBackend default ajax timeout(in millisecond)
			 *  default is 30000
			 */
			ajaxTimeout : 200000,

			/**
			 * config $httpBackend use ajax queue mode, in queue mode
			 * all ajax request will execute one by one
			 *  ajaxQueueSize is queue max length, 0 means no queue
			 *  ajaxAborted use for ajax abort or not if duplicated request
			 * default is (5, false)
			 */
			ajaxQueueSize : 5,
			ajaxAborted : false,
			/**
			 * config $httpBackend beforSend and afterReceived callback
			 * typically use for ajax indicator
			 * NOTE: you should config it for ajax indicator
			 */
			beforeSend : function() {
				// beforeSend
					if (NativeCall.timer) {
						clearTimeout(NativeCall.timer);
						NativeCall.timer = null;
					}

					if (!NativeCall.ShowMasking) {
						NativeCall.ShowMasking = true;
						NativeCall.showMask();
					}
			},
			afterReceived : function() {
				// afterReceived
				if(NativeCall.timer){
                    clearTimeout(NativeCall.timer);
                    NativeCall.timer = null;
				}
				if (!NativeCall.timer){
					NativeCall.timer = setTimeout(function() {
						NativeCall.timer = null;
						NativeCall.ShowMasking = false;
						NativeCall.hideMask();
					}, 1000);
					NativeCall.closeSplashScreen();
				}
			}
		});
	}

	//Validation配置
	configValidation.$inject = ['$validationProvider'];
	function configValidation($validationProvider) {
		/**
		 * register validation type, validation is use in data-binding, so it is
		 * bi-direction include parse and format, so validation function could
		 * registerred in 2 modes:
		 * 1.
		 *  $validationProvider.register('validator', function(value) {
		 *	  // return converted value, or undefined for invalid value
		 *  });
		 * 2.
		 *  $validationProvider.register('validator', {
		 *	  parse : function(value) {
		 *		// return converted value, or undefined for invalid value
		 *	  },
		 *	  format : function(value) {
		 *		// return converted value, or undefined for invalid value
		 *	  }
		 *  });
		 */

	}

	//共9个配置
	mod.config(configLog);
	mod.config(configBrowser);
	mod.config(configTargets);
	mod.config(configCompile);
	mod.config(configRootScope);
	mod.config(configRemote);
	mod.config(configHttp);
	mod.config(configHttpBackend);
	mod.config(configValidation);

	/************************************************
	 * config service instance function
	 ************************************************/
	//runTargets
	runTargets.$inject = ['$targets', '$rootScope', '$transitions'];
	function runTargets($targets, $rootScope, $transitions) {
		vx.forEach($transitions.types, function(value, key) {
			$targets.transition(value, (function() {
				var type = $transitions.types[value];
				return function(oldEl, newEl, remove, back) {
					$transitions.runTransition(type, oldEl, newEl, remove, back);
				};
			})());
		});
	}

	//runRootScope
	runRootScope.$inject = [ "$rootScope", "$nativeCall", "$goto", "$timeout", "$http", "$targets", "$parse" , "$os"];
	function runRootScope( $rootScope, $nativeCall, $goto, $timeout, $http, $targets, $parse, $os) {
		//图形验证码获取 
		//	$rootScope.hnnxImageUrl = "https://18.1.99.52/";
			var intervalSeconds = 60;
			//android 弹出短信验证码内容
			$nativeCall.getSystemVar(function(data){
				if(data.smsTimeout){
					intervalSeconds = data.smsTimeout;
				}
			});

		//初始化发送验证码按钮
		$rootScope.initGetToken = function(id, flagStr){
			$("#content #"+id).addClass("hnb-btn-disabled");
			var scope = this;
			//方便测试，时间改为180秒，测试完改为60秒
//			var intervalSeconds = 60;
			$timeout.cancel($rootScope.timer);
			$rootScope.timerFlag = false;
			$rootScope.setTimer(intervalSeconds, id, flagStr);
//			$("#content #" + id).attr("disabled", "true");
//			if(flagStr){
//				$parse(flagStr).assign(scope, "true"); //按钮disabled
//			}
			if($os.android) {
				NativeCall.getSmsCode(function(smsCode){
					scope.verificationCode = smsCode;
					scope.$apply();
				});
			}
		} ;
		var i=0;
		//获取短信验证码
		$rootScope.getToken = function(id, flagStr) {
			var scope = this;
			//方便测试，时间改为180秒，测试完改为60秒
//			var intervalSeconds = $nativeCall.getSystemVar(function(){});
			//var intervalSeconds = 180;
			if($rootScope.timerFlag){
				return;
			}
			$("#content #"+id).addClass("hnb-btn-disabled");
			//发交易获取短信验证码
			$http.post("JsonTransServlet", {
				"oldTempId":scope.tempId,
				"transCode":"P888887"
			}).success(function(data, status, headers, config) {
				var returnCode = data.returnCode;
				if(returnCode && returnCode == "000000000"){
					//页面赋值
					scope.serialNumber = data.serialNumber;
					scope.tempId = data.tempId;
					//android 弹出短信验证码内容
					if($os.android) {
						NativeCall.getSmsCode(function(smsCode){
							scope.verificationCode = smsCode;
							scope.$apply();
						});
					}
					
//					$("#content #" + id).attr("disabled", "true");
//					if(flagStr){
//						$parse(flagStr).assign(scope, "true");
//					}
					$rootScope.setTimer(intervalSeconds, id, flagStr);
				}else{
					scope.toast(data.returnMsg);
					$targets("content","#-1");
				}
			}).error(function(data, status, headers, config) {
				scope.toast(data.returnMsg);
			});
		};
		
		//设置时间方法。60秒
		$rootScope.setTimer = function(value, id, flagStr) {
		
			value = value - 1;
			var showText =  value ;
			$("#content #" + id).text(showText);
			if (value >= 0) {
				$rootScope.timerFlag = true;
				$rootScope.timer = $timeout(function() {
					$rootScope.setTimer(value, id, flagStr);
				}, 1000);
			} else {
				$rootScope.clearTimeouts(id, flagStr);
			}
		};
		//当时间为零，重新显示“获取短信验证码”
		$rootScope.timer = null;
		$rootScope.timerFlag = false;
		$rootScope.clearTimeouts = function(id, flagStr) {
			$("#content #"+id).html("点击获取");
			$("#content #" + id).removeAttr("disabled");
			$("#content #"+id).removeClass("hnb-btn-disabled");
			var scope = this;
//			scope.toast("销毁计时器 -"+id+"-"+flagStr+"-" + $rootScope.timer != null);
			scope.textNo = null;
			if ($rootScope.timer != null) {
				$timeout.cancel($rootScope.timer);
				$rootScope.timerFlag = false;

//				$("#content #" + id).removeAttr("disabled");
//				$("#content #" + id).html("点击获取");
//				if(flagStr){
//					$parse(flagStr).assign(scope, false); //按钮disabled
//				}
				
				
			}
			$rootScope.timer = null;
		};
		//新建转账btn
		$rootScope.BuildNewTransfer = function(){
			//行内转账
			if($rootScope.ThisPageId == "行内转账"){
				var scope = this;
				scope.BuildNewInTransfer();
			}else if ($rootScope.ThisPageId == "跨行转账"){//跨行转账
				var scope = this;
				scope.BuildNewOutTransferMethod();
			}
		}
		
		//自助签约btn
		$rootScope.ToSelfSignMethod = function(){
			var scope = this;
			scope.ToSelfSign();
		}

		$rootScope.alert = function(message) {
			$nativeCall.alert(message);
		};
 		
		$rootScope.confirm = function(message, okBack, cancleBack) {
			var scope = this;
			var cfmMessage = {
				"title" : "确认",
				"message" : message,
				"positiveText" : "确定",
				"negativeText" : "取消"
			};
			$nativeCall.confirm.call(this, function(confirm) {
				if (confirm == "Yes") {
					if ( typeof okBack === "function") {
						scope.$apply(okBack());
					}
				} else {
					if ( typeof cancleBack === "function") {
						scope.$apply(cancleBack());
					}
				}
			}, vx.toJson(cfmMessage));
		};

		$rootScope.toast = function(errMessage) {
			$nativeCall.toast(errMessage);
		};

		$rootScope.authenticate = function(callback) {
			$nativeCall.authenticate(callback);
		};
		//html页面跳转
		$rootScope.loadPage = function(sourse,targets){
			$nativeCall.pages.push(sourse);
			$nativeCall.history = [];
			$targets("content",targets);
		}

		$rootScope.finishWeb = function(param) {
			if(param === 'login'){
				$rootScope.closeBank();
			}else{
				$nativeCall.finishWeb();
			}
			
		};
		
		$rootScope.closeBank = function() {
			$rootScope.toast("close2");
			$nativeCall.closeBank();
		};

		$rootScope.gotoUrl = function(url) {
			$goto(url);
		};
		
		$rootScope.openWebsite = function(url) {
			$nativeCall.openWebsite(url);
		};

		$rootScope.showHelp = function() {
			$nativeCall.history.push(function() {
				$nativeCall.backTransition();
				$timeout(function() {
					$('#Help').hide();
					$("#HelpImg").show();
					$('#content').css("display", "block");
					$nativeCall.startTransition();
				}, 50);
			});
			$nativeCall.forWardTransition();
			$timeout(function() {
				$('#content').css("display", "none");
				$('#Help').show();
				$("#HelpImg").hide();
				window.scrollTo(0, 0);
				$nativeCall.startTransition();
			}, 156);
		};
		
		/**通知客户端获取公钥**/
		$rootScope.publicKeyModulus = function(callback){
			$nativeCall.publicKeyModulus(callback);
		};
		
		/**重新登录**/
		$rootScope.reLogin = function(){
			$nativeCall.reLogin(function(data){
				
			});
		};
		
		/**用户登出**/
		$rootScope.userLogout = function(){
			$nativeCall.userLogout(function(logoutData){
			});
		};
		
		/**加载密码控件**/
		$rootScope.initPwd = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticate(function(data){
			
			//mode.assign(scope, data);
			//scope.$apply();
			
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
				//scope.transPwd="";
			//	var s="";
			//	for(var i=0;i<data;i++){
			//		s+="*";
			//	}
			//	scope.transPwd=data;
				
			});
		};
		
		/**加载确认密码控件**/
		$rootScope.initConfirmPwd = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticateConfirm(function(data){
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
			});
		};
		
		/**加载密码控件V1**/
		$rootScope.initPwdV1 = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticateV1(function(data){
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
			});
		};
		
		/**加载密码控件V2 汽车票取票密码**/
		$rootScope.initPwdV2 = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticateV2(function(data){
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
			});
		};
		
		/**加载确认密码控件V1 **/
		$rootScope.initConfirmPwdV1 = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticateConfirmV1(function(data){
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
			});
		};
		
		/**加载确认密码控件V2 汽车票取票密码确认**/
		$rootScope.initConfirmPwdV2 = function(vmodel){
			vx.element("#content #"+vmodel).blur();
			var scope = this;
			var mode = $parse(vmodel)
			$nativeCall.authenticateConfirmV2(function(data){
				mode.assign(scope, (vx.fromJson(data)).TrsPassword) ;
				scope.$apply();
			});
		};
		
		/**验证支付密码、确认支付密码**/
		$rootScope.bothPwdVerify = function(callback){
			$nativeCall.bothPwdVerify(callback);
		};
		
		/**验证取款密码、确认取款密码**/
		$rootScope.bothPwdVerifyV1 = function(callback){
			$nativeCall.bothPwdVerifyV1(callback);
		};
		
		/**验证取票密码、确认取票密码**/
		$rootScope.bothPwdVerifyV2 = function(callback){
			$nativeCall.bothPwdVerifyV2(callback);
		};
		
		/**获取用户信息**/
		$rootScope.getUserInfo = function(callback){
			$nativeCall.getUserInfo(callback);
		};
		
		/**返回首页，弹出信息**/
		$rootScope.finishWebWithMessage = function(msg) {
		//	if(param === 'login'){
		//		$rootScope.closeBank();
		//	}else{
			//	$timeout(function() {
					$nativeCall.finishWebWithMessage(msg);
			//	}, 1000);
		//	}
		};
	}

	//runNativeCall
	runNativeCall.$inject = ['$nativeCall'];
	function runNativeCall($nativeCall) {
		window.NativeCall = $nativeCall;
	}


	runOS.$inject = ['$os'];
	function runOS($os) {
		window.OS = $os;
	}

	mod.run(runTargets);
	mod.run(runOS);
	mod.run(runRootScope);
	mod.run(runNativeCall);

})(window, window.vx, window.jQuery);

